<?php
/**
 * Processes Instant Payment Notifiations from CLICK.
 */

/**
 * Processes prepare request from CLICK.
 */
function uc_click_prepare_ipn() {
  $id = uc_click_webhook_log();
  $result = uc_click_webhook_errors();
  $result += array(
    'click_trans_id' => $_POST['click_trans_id'],
    'merchant_trans_id' => $_POST['merchant_trans_id'],
    'merchant_prepare_id' => $id,
  );

  if ($result['error'] === '0') {
    // set order status to on-hold
    $order = uc_order_load($_POST['merchant_trans_id']);
    $order->order_status = 'processing';
    uc_order_save($order);
  }

  drupal_json_output($result);
}

/**
 * Processes complete request from CLICK.
 */
function uc_click_complete_ipn() {
  $id = uc_click_webhook_log();
  $result = uc_click_webhook_errors();
  $result += array(
    'click_trans_id' => $_POST['click_trans_id'],
    'merchant_trans_id' => $_POST['merchant_trans_id'],
    'merchant_confirm_id' => $id,
  );

  if ($_POST['error'] < 0) {
    $order = uc_order_load($_POST['merchant_trans_id']);
    $order->order_status = 'canceled';
    uc_order_save($order);
  }

  if ($result['error'] === '0') {
    // complete payment
    $order = uc_order_load($_POST['merchant_trans_id']);
    $comment = t('CLICK transaction ID: @txn_id', array('@txn_id' => $_POST['click_trans_id']));
    uc_payment_enter($_POST['merchant_trans_id'], 'uc_click', $_POST['amount'], $order->uid, NULL, $comment);
    uc_cart_complete_sale($order);
//    unset($_SESSION['uc_checkout'][$order->order_id], $_SESSION['cart_order']);
  }

  drupal_json_output($result);
}


function uc_click_webhook_log() {
  return db_insert('uc_click_ipn')
    ->fields(array(
      'click_trans_id' => $_POST['click_trans_id'],
      'service_id' => $_POST['service_id'],
      'click_paydoc_id' => $_POST['click_paydoc_id'],
      'merchant_trans_id' => $_POST['merchant_trans_id'],
      'merchant_prepare_id' => $_POST['action'] > 0 ? $_POST['merchant_prepare_id'] : NULL,
      'amount' => $_POST['amount'],
      'action' => $_POST['action'],
      'error' => $_POST['error'],
      'error_note' => $_POST['error_note'],
      'sign_time' => $_POST['sign_time'],
      'sign_string' => $_POST['sign_string'],
    ))
    ->execute();
}

function uc_click_get_log($id) {
  return db_query('SELECT * FROM {uc_click_ipn} WHERE id = :id', array(':id' => $id))->fetchAssoc();
}

function uc_click_webhook_errors() {
  $click_trans_id = $_POST['click_trans_id'];
  $service_id = $_POST['service_id'];
  $click_paydoc_id = $_POST['click_paydoc_id'];
  $order_id = $_POST['merchant_trans_id'];
  $amount = $_POST['amount'];
  $action = $_POST['action'];
  $error = $_POST['error'];
  $error_note = $_POST['error_note'];
  $sign_time = $_POST['sign_time'];
  $sign_string = $_POST['sign_string'];
  $merchant_prepare_id = $action > 0 ? $_POST['merchant_prepare_id'] : '';

  if (!isset(
    $click_trans_id,
    $service_id,
    $click_paydoc_id,
    $amount,
    $action,
    $error,
    $error_note,
    $sign_time,
    $sign_string
  ) || ($action > 0 && !isset($merchant_prepare_id))) {
    return array(
      'error'      => '-8',
      'error_note' => t('Error in request from click')
    );
  }

  $signString = $click_trans_id .
    $service_id .
    variable_get('uc_click_secret_key', '') .
    $order_id .
    $merchant_prepare_id .
    $amount .
    $action .
    $sign_time;
  $signString = md5($signString);
  if ($signString !== $sign_string) {
    return array(
      'error'      => '-1',
      'error_note' => t('SIGN CHECK FAILED!')
    );
  }

  if (!($action == 0 || $action == 1)) {
    return array(
      'error'      => '-3',
      'error_note' => t('Action not found')
    );
  }

  $order = uc_order_load($order_id);

  if (!$order) {
    return array(
      'error'      => '-5',
      'error_note' => t('User does not exist')
    );
  }

  if ($order->order_status == 'completed' || $order->order_status == 'payment_received') {
    return array(
      'error'      => '-4',
      'error_note' => t('Already paid')
    );
  }

  if (abs((float)$amount - $order->order_total) > 0.01) {
    return array(
      'error'      => '-2',
      'error_note' => t('Incorrect parameter amount')
    );
  }

  if ($action == 1) {
    $log = uc_click_get_log($merchant_prepare_id);
    if (!$log) {
      return array(
        'error'      => '-6',
        'error_note' => t('Transaction does not exist')
      );
    }
  }

  if ($order->order_status == 'canceled' || $error < 0) {
    return array(
      'error'      => '-9',
      'error_note' => t('Transaction cancelled')
    );
  }

  return array(
    'error'      => '0',
    'error_note' => t('Success')
  );
}


/**
 * Handles a complete CLICK sale.
 */
function uc_click_complete($order) {
  // If the order ID specified in the return URL is not the same as the one in
  // the user's session, we need to assume this is either a spoof or that the
  // user tried to adjust the order on this side while at CLICK. If it was a
  // legitimate checkout, the IPN will still come in from CLICK so the order
  // gets processed correctly. We'll leave an ambiguous message just in case.

  if (!isset($_SESSION['cart_order']) || intval($_SESSION['cart_order']) != $order->order_id) {
    drupal_set_message(t('Thank you for your order! CLICK will notify us once your payment has been processed.'));
    drupal_goto('cart');
  }

  // Ensure the payment method is CLICK.
  if ($order->payment_method != 'uc_click') {
    drupal_goto('cart');
  }

  // This lets us know it's a legitimate access of the complete page.
  $_SESSION['uc_checkout'][$_SESSION['cart_order']]['do_complete'] = TRUE;
  drupal_goto('cart/checkout/complete');
}

/**
 * Handles a canceled CLICK sale.
 */
function uc_click_cancel() {
  unset($_SESSION['cart_order']);

  drupal_set_message(t('Your CLICK payment was canceled. Please feel free to continue shopping or contact us for assistance.'));

  drupal_goto(variable_get('uc_click_cancel_return_url', 'cart'));
}
